extern crate renbin;

use clap::Parser;
use renbin::{
    FileDescriptor,
    engine::{Engine, fs::FsEngine, rentry::RentryEngine},
};
use std::{
    fs::{read, remove_file},
    path::PathBuf,
};

#[derive(Parser, Debug)]
#[command(version, about, long_about = None)]
struct Args {
    #[arg(short = 'd', long = "decode", action)]
    decode: bool,
    #[arg(short = 'x', long = "delete", action)]
    delete: bool,
    #[arg(short = 'i', long = "input")]
    path: String,
    #[arg(short = 'e', long = "engine")]
    #[clap(default_value = "fs")]
    engine: String,
}

#[tokio::main]
async fn main() {
    let args = Args::parse();

    if args.delete {
        // delete
        let path = PathBuf::from(args.path);

        if args.engine == "fs" {
            FsEngine
                .delete(FileDescriptor::read(path.clone().into()))
                .await
                .expect("failed to reconstruct file");
        } else if args.engine == "rentry" {
            let mut engine = RentryEngine::new();
            engine.auth().await.expect("failed to extract csrf token");

            engine
                .delete(FileDescriptor::read(path.clone().into()))
                .await
                .expect("failed to delete file");
        } else {
            panic!("unknown engine type");
        };

        remove_file(path).expect("failed to delete file descriptor");
        return;
    }

    if !args.decode {
        // encode
        let pathbuf = PathBuf::from(args.path);

        if args.engine == "fs" {
            FsEngine
                .process(
                    pathbuf.file_name().unwrap().to_str().unwrap().to_string(),
                    read(pathbuf).expect("failed to read file"),
                )
                .await
                .expect("failed to process file")
        } else if args.engine == "rentry" {
            let mut engine = RentryEngine::new();
            engine.auth().await.expect("failed to extract csrf token");

            engine
                .process(
                    pathbuf.file_name().unwrap().to_str().unwrap().to_string(),
                    read(pathbuf).expect("failed to read file"),
                )
                .await
                .expect("failed to process file")
        } else {
            panic!("unknown engine type");
        };
    } else {
        // decode
        if args.engine == "fs" {
            FsEngine
                .reconstruct(FileDescriptor::read(PathBuf::from(args.path).into()))
                .await
                .expect("failed to reconstruct file");
        } else if args.engine == "rentry" {
            let mut engine = RentryEngine::new();
            engine.auth().await.expect("failed to extract csrf token");

            engine
                .reconstruct(FileDescriptor::read(PathBuf::from(args.path).into()))
                .await
                .expect("failed to reconstruct file");
        } else {
            panic!("unknown engine type");
        };
    }
}
